@@ -16,6 +16,7 @@ import importlib
 import warnings
 
 from keystoneauth1 import adapter
+import keystoneauth1.exceptions.catalog
 from keystoneauth1 import plugin
 from keystoneauth1 import session
 import requestsexceptions
@@ -70,7 +71,7 @@ def _make_key(key, service_type):
 class CloudConfig(object):
     def __init__(self, name, region, config,
                  force_ipv4=False, auth_plugin=None,
-                 openstack_config=None):
+                 openstack_config=None, session_constructor=None):
         self.name = name
         self.region = region
         self.config = config
@@ -79,6 +80,7 @@ class CloudConfig(object):
         self._auth = auth_plugin
         self._openstack_config = openstack_config
         self._keystone_session = None
+        self._session_constructor = session_constructor or session.Session
 
     def __getattr__(self, key):
         """Return arbitrary attributes."""
@@ -101,6 +103,10 @@ class CloudConfig(object):
     def __ne__(self, other):
         return not self == other
 
+    def set_session_constructor(self, session_constructor):
+        """Sets the Session constructor."""
+        self._session_constructor = session_constructor
+
     def get_requests_verify_args(self):
         """Return the verify and cert values for the requests library."""
         if self.config['verify'] and self.config['cacert']:
@@ -196,7 +202,7 @@ class CloudConfig(object):
                     " since verify=False".format(
                         cloud=self.name, region=self.region))
             requestsexceptions.squelch_warnings(insecure_requests=not verify)
-            self._keystone_session = session.Session(
+            self._keystone_session = self._session_constructor(
                 auth=self._auth,
                 verify=verify,
                 cert=cert,
@@ -246,11 +252,18 @@ class CloudConfig(object):
         if service_key == 'identity':
             endpoint = session.get_endpoint(interface=plugin.AUTH_INTERFACE)
         else:
-            endpoint = session.get_endpoint(
-                service_type=self.get_service_type(service_key),
-                service_name=self.get_service_name(service_key),
-                interface=self.get_interface(service_key),
-                region_name=self.region)
+            args = {
+                'service_type': self.get_service_type(service_key),
+                'service_name': self.get_service_name(service_key),
+                'interface': self.get_interface(service_key),
+                'region_name': self.region
+            }
+            try:
+                endpoint = session.get_endpoint(**args)
+            except keystoneauth1.exceptions.catalog.EndpointNotFound:
+                self.log.warning("Keystone catalog entry not found (%s)",
+                                 args)
+                endpoint = None
         return endpoint
 
     def get_legacy_client(
