@@ -27,6 +27,11 @@ from os_client_config import exceptions
 from os_client_config.tests import base
 
 
+def prompt_for_password(prompt=None):
+    """Fake prompt function that just returns a constant string"""
+    return 'promptpass'
+
+
 class TestConfig(base.TestCase):
 
     def test_get_all_clouds(self):
@@ -102,6 +107,7 @@ class TestConfig(base.TestCase):
         self.assertEqual('123456789', cc.auth['project_domain_id'])
         self.assertNotIn('domain_id', cc.auth)
         self.assertNotIn('domain-id', cc.auth)
+        self.assertNotIn('domain_id', cc)
 
     def test_get_one_cloud_domain_scoped(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -118,6 +124,7 @@ class TestConfig(base.TestCase):
         self.assertEqual('awesome-domain', cc.auth['user_domain_id'])
         self.assertEqual('awesome-domain', cc.auth['project_domain_id'])
         self.assertNotIn('domain_id', cc.auth)
+        self.assertNotIn('domain_id', cc)
 
     def test_get_one_cloud_with_hyphenated_project_id(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -192,11 +199,19 @@ class TestConfig(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
         cc = c.get_one_cloud('_test-cloud-networks_')
         self.assertEqual(
-            ['a-public', 'another-public'], cc.get_external_networks())
+            ['a-public', 'another-public', 'split-default'],
+            cc.get_external_networks())
         self.assertEqual(
-            ['a-private', 'another-private'], cc.get_internal_networks())
+            ['a-private', 'another-private', 'split-no-default'],
+            cc.get_internal_networks())
         self.assertEqual('another-private', cc.get_nat_destination())
         self.assertEqual('another-public', cc.get_default_network())
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-no-default'],
+            cc.get_external_ipv4_networks())
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-default'],
+            cc.get_external_ipv6_networks())
 
     def test_get_one_cloud_no_networks(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -373,6 +388,95 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
 
+    def test_get_one_cloud_precedence(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        kwargs = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'authpass',
+                'project-id': 'testproject',
+                'auth_url': 'http://example.com/v2',
+            },
+            'region_name': 'kwarg_region',
+            'password': 'ansible_password',
+            'arbitrary': 'value',
+        }
+
+        args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            password='argpass',
+            project_name='project',
+            region_name='region2',
+            snack_type='cookie',
+        )
+
+        options = argparse.Namespace(**args)
+        cc = c.get_one_cloud(
+            argparse=options, **kwargs)
+        self.assertEqual(cc.region_name, 'region2')
+        self.assertEqual(cc.auth['password'], 'authpass')
+        self.assertEqual(cc.snack_type, 'cookie')
+
+    def test_get_one_cloud_precedence_osc(self):
+        c = config.OpenStackConfig(
+            config_files=[self.cloud_yaml],
+            vendor_files=[self.vendor_yaml],
+        )
+
+        kwargs = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'authpass',
+                'project-id': 'testproject',
+                'auth_url': 'http://example.com/v2',
+            },
+            'region_name': 'kwarg_region',
+            'password': 'ansible_password',
+            'arbitrary': 'value',
+        }
+
+        args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            password='argpass',
+            project_name='project',
+            region_name='region2',
+            snack_type='cookie',
+        )
+
+        options = argparse.Namespace(**args)
+        cc = c.get_one_cloud_osc(
+            argparse=options,
+            **kwargs
+        )
+        self.assertEqual(cc.region_name, 'region2')
+        self.assertEqual(cc.auth['password'], 'argpass')
+        self.assertEqual(cc.snack_type, 'cookie')
+
+    def test_get_one_cloud_precedence_no_argparse(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        kwargs = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'authpass',
+                'project-id': 'testproject',
+                'auth_url': 'http://example.com/v2',
+            },
+            'region_name': 'kwarg_region',
+            'password': 'ansible_password',
+            'arbitrary': 'value',
+        }
+
+        cc = c.get_one_cloud(**kwargs)
+        self.assertEqual(cc.region_name, 'kwarg_region')
+        self.assertEqual(cc.auth['password'], 'authpass')
+        self.assertIsNone(cc.password)
+
     def test_get_one_cloud_just_argparse(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -649,8 +753,8 @@ class TestConfigArgparse(base.TestCase):
         opts, _remain = parser.parse_known_args(args)
         self.assertEqual(opts.os_service_type, 'network')
         self.assertEqual(opts.os_endpoint_type, 'admin')
-        self.assertEqual(opts.os_network_service_type, None)
-        self.assertEqual(opts.os_network_api_version, None)
+        self.assertIsNone(opts.os_network_service_type)
+        self.assertIsNone(opts.os_network_api_version)
         self.assertEqual(opts.network_api_version, '4')
         cloud = c.get_one_cloud(argparse=opts, verify=False)
         self.assertEqual(cloud.config['service_type'], 'network')
@@ -672,12 +776,12 @@ class TestConfigArgparse(base.TestCase):
             parser, args, ['compute', 'network', 'volume'])
         opts, _remain = parser.parse_known_args(args)
         self.assertEqual(opts.os_network_service_type, 'badtype')
-        self.assertEqual(opts.os_compute_service_type, None)
-        self.assertEqual(opts.os_volume_service_type, None)
+        self.assertIsNone(opts.os_compute_service_type)
+        self.assertIsNone(opts.os_volume_service_type)
         self.assertEqual(opts.os_service_type, 'compute')
         self.assertEqual(opts.os_compute_service_name, 'cloudServers')
         self.assertEqual(opts.os_endpoint_type, 'admin')
-        self.assertEqual(opts.os_network_api_version, None)
+        self.assertIsNone(opts.os_network_api_version)
         self.assertEqual(opts.network_api_version, '4')
         cloud = c.get_one_cloud(argparse=opts, verify=False)
         self.assertEqual(cloud.config['service_type'], 'compute')
@@ -688,6 +792,38 @@ class TestConfigArgparse(base.TestCase):
         self.assertNotIn('http_timeout', cloud.config)
 
 
+class TestConfigPrompt(base.TestCase):
+
+    def setUp(self):
+        super(TestConfigPrompt, self).setUp()
+
+        self.args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            project_name='project',
+            # region_name='region2',
+            auth_type='password',
+        )
+
+        self.options = argparse.Namespace(**self.args)
+
+    def test_get_one_cloud_prompt(self):
+        c = config.OpenStackConfig(
+            config_files=[self.cloud_yaml],
+            vendor_files=[self.vendor_yaml],
+            pw_func=prompt_for_password,
+        )
+
+        # This needs a cloud definition without a password.
+        # If this starts failing unexpectedly check that the cloud_yaml
+        # and/or vendor_yaml do not have a password in the selected cloud.
+        cc = c.get_one_cloud(
+            cloud='_test_cloud_no_vendor',
+            argparse=self.options,
+        )
+        self.assertEqual('promptpass', cc.auth['password'])
+
+
 class TestConfigDefault(base.TestCase):
 
     def setUp(self):
@@ -820,7 +956,9 @@ class TestBackwardsCompatibility(base.TestCase):
         expected = {
             'networks': [
                 {'name': 'private', 'routes_externally': False,
-                 'nat_destination': False, 'default_interface': False},
+                 'nat_destination': False, 'default_interface': False,
+                 'routes_ipv4_externally': False,
+                 'routes_ipv6_externally': False},
             ]
         }
         self.assertEqual(expected, result)
